import { 
    Plugin, icons, toWidget, Command, FileRepository, FileDialogButtonView, 
    type Editor, type ViewContainerElement, type Element
} from 'ckeditor5';


export default class VideoUpload extends Plugin {
    static readonly pluginName: string = 'VideoUpload';
    
    static get requires() {
        return [FileRepository, VideoInsertCommand];
    }

    init() {
        const editor = this.editor;
        const t = editor.t;

        editor.commands.add('videoInsert', new VideoInsertCommand(editor));

        // Register a new toolbar button
        editor.ui.componentFactory.add('videoUpload', locale => {
            const view = new FileDialogButtonView(locale);
            view.set({
                acceptedType: 'video/*',
                label: t('Upload Video'),
                icon: icons.check,
                withText: false,
                tooltip: true
            });

            // When a file is selected, execute the upload command
            view.on('done', (evt, files: FileList) => {
                const file = files[0]!;
                if (file) {
                    editor.execute('videoInsert', { file });
                }
            });

            return view;
        });
    }
}

// Command to handle video insertion after upload
class VideoInsertCommand extends Command {
    constructor(editor: Editor) {
        super(editor);
    }

    execute(options: { file: File }) {
        const editor = this.editor;
        const fileRepository = editor.plugins.get('FileRepository');
        if (!fileRepository)
            return;

        // Create a loader for the file
        const loader = fileRepository.createLoader(options.file);
        if (!loader)
            return;

        loader.upload()
            .then(response => {
                const url = (response as { default: string }).default;
                editor.model.change(writer => {
                    // TODO upcast?
                    const videoElement = writer.createElement('video', { src: url, controls: true });
                    editor.model.insertContent(videoElement, editor.model.document.selection);
                });
            })
            .catch((error: Error) => {
                console.error('Video upload failed:', error);
            });
    }
}

function configureVideo(editor: Editor) {
    editor.model.schema.register('video', {
        inheritAllFrom: '$blockObject',
        allowAttributes: ['src', 'controls', 'width']
    });

    editor.conversion.for('upcast').elementToElement({
        view: {
            name: 'video',
            attributes: ['src', 'controls' ]
        },
        model: (viewElement, { writer }) => {
            const src = viewElement.getAttribute('src')!;
            return writer.createElement('videoElement', { src, controls: true });
        }
    });

    editor.conversion.for('dataDowncast').elementToElement({
        model: 'video',
        view: (modelElement, { writer }) => {
            const src = modelElement.getAttribute('src');
            const width = modelElement.getAttribute('width');
            return writer.createContainerElement('video', {
                src,
                controls: 'controls',
                width: width || "750px"
            });
        }
    });

    editor.conversion.for('editingDowncast').elementToElement({
        model: 'video',
        view: (modelElement, { writer }) => {
            const src = modelElement.getAttribute('src');
            const videoView = writer.createContainerElement('video', { src, controls: true });
            const widget = toWidget(videoView, writer);

            // Add resizing functionality to the widget
            setupResizer(widget, editor, modelElement);

            return widget;
        }
    });
}

function setupResizer(widget: ViewContainerElement, editor: Editor, modelElement: Element) {
    editor.plugins.get('WidgetResize').attachTo({
        unit: 'px',
        modelElement: modelElement,
        viewElement: widget,
        editor,

        // Specify where the resize handles should be rendered
        getHandleHost(domWidgetElement: HTMLElement): HTMLElement {
            return domWidgetElement;
        },

        // Specify the resize host (the element that will be resized)
        getResizeHost(domWidgetElement: HTMLElement): HTMLElement {
            return domWidgetElement;
        },

        // Handle resizing event
        onCommit(newSize: string) {
            editor.model.change(writer => {
                writer.setAttribute('width', newSize, modelElement);
            });
        }
    });
}

export { VideoUpload, configureVideo };
